# -*- coding: utf-8 -*-

import os, logging, sys
from functools import partial

from qgis.gui import *
from qgis.core import *
from qgis.utils import iface
from PyQt5 import QtGui, QtWidgets, uic, QtXml
from PyQt5.QtCore import *

from .rdi_param import rdiParam
from .rdi_tools import quote
from .rdi_layer import rdiLayer, rdiLayersCollection

memoRdiLegend = {}


class rdiLegend():
    def __init__(self, dock, layout):
        self.dock = dock
        self.items = {}
        self.ori = {}
        self.layout = layout
    
    def setlayout(self, layout):
        self.layout = layout
    
    def add(self, name, catalog, rdi):
        self.ori[rdi] = name
        if not name in self.items:
            item = rdiLegendItem(name, catalog, self.layout)
            item.clicked.connect(partial(self.makeContext, item))
            self.items[name] = item
        else:
            item = self.items[name]
            item.show()
        return item
    
    def makeContext(self, item=None):
        for rdi,name in self.ori.items():
            if not item:
                item = self.items[name]
            if item.name==name:
                rdi.setContextFromLegend(item)
        self.dock.actuLayersAlea()
        self.dock.actuLayersEnjeu()
    
    def remove(self, rdi):
        if rdi in self.ori:
            del self.ori[rdi]
            self.clean()
        
    def clean(self):
        lst = self.ori.values()
        for name,item in self.items.items():
            if not name in lst:
                item.hide()


class rdiLegendItem(QObject):
    
    clicked = pyqtSignal()
    
    def __init__(self, name, catalog, layout):
        QObject.__init__(self)
        self.name = name
        self.catalog = catalog
        self.layout = layout
        self.elts = []
        self.draw()
        
    def draw(self):
        if 'list' in self.catalog:
            if not self.name in memoRdiLegend:
                memoRdiLegend[self.name] = {}
            # l = QtWidgets.QLabel()
            # self.layout.addWidget(l)
            # self.space = l
            gb = QtWidgets.QGroupBox(self.name)
            self.layout.addWidget(gb)
            lgb = QtWidgets.QVBoxLayout(gb) 
            lgb.setSpacing(0)
            for i,e in enumerate(self.catalog['list']):
                elt = rdiLegendItemElt(i, e, lgb)
                if i in memoRdiLegend[self.name] and memoRdiLegend[self.name][i]==False:
                    elt.setChecked(False)
                elt.clicked.connect(self.click)
                self.elts.append(elt)  
            self.group = gb
    
    def click(self, ind, checked):
        memoRdiLegend[self.name][ind] = checked
        self.clicked.emit()
    
    def hide(self):
        # self.space.hide()
        self.group.hide()
        
    def show(self):
        # self.space.show()
        self.group.show()
    


class rdiLegendItemElt(QObject):
    
    clicked = pyqtSignal(int, bool)
    
    def __init__(self, ind, elt, layout):
        QObject.__init__(self)
        self.layout = layout
        self.elt = elt
        self.ind = ind
        self.draw()
        
    def draw(self):
        hbox = QtWidgets.QHBoxLayout()
        hbox.setSpacing(5)
        self.layout.addLayout(hbox)
        l = QtWidgets.QLabel()
        l.setStyleSheet(f"background-color:{self.elt['color']};")
        l.setFixedSize(17, 17)
        hbox.addWidget(l)
        cb = QtWidgets.QCheckBox(self.elt['label'])
        cb.setChecked(True)
        cb.stateChanged.connect(self.click)
        hbox.addWidget(cb)
        self.cb = cb
        
    def click(self):
        self.clicked.emit(self.ind, self.cb.isChecked())
    
    def setChecked(self, bool):
        self.cb.setChecked(bool)
    
    def isChecked(self):
        return self.cb.isChecked()
                
   


class rdiAlleviate(QObject):
    def __init__(self, parent):
        QObject.__init__(self)
        self.dock = parent
        self.excludes = {}
         
    def openWindow(self, code):
        buttons = QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel
        dialog = QgsDialog(iface.mainWindow(),
                  fl=Qt.WindowFlags(),
                  buttons=buttons,
                  orientation=Qt.Horizontal)
        dialog.setWindowTitle(f"Sélectionner les {self.dock.param.get(code)}") 
        coll = rdiLayersCollection(self.dock, code)
        w = QtWidgets.QWidget(dialog)       
        w.setMinimumWidth(250)
        dialog.layout().addWidget(w)
        gbox = QtWidgets.QVBoxLayout(w)
        self.makeLayerSelectTitle(gbox, coll)
        sc = QtWidgets.QScrollArea(dialog)
        sc.setWidgetResizable(True)
        dialog.layout().addWidget(sc)
        w = QtWidgets.QWidget()
        gbox = QtWidgets.QVBoxLayout()
        gbox.setSpacing(0)
        gbox.setAlignment(Qt.AlignTop)
        w.setLayout(gbox)
        sc.setWidget(w)
        self.makeLayerSelectList(gbox, coll)
        dialog.accepted.connect(partial(self.rebuiltList, coll))
        self.setInfos(coll)
        dialog.show()

    def makeLayerSelectTitle(self, layout, coll):
        hbox = QtWidgets.QHBoxLayout()
        layout.addLayout(hbox)
        l = QtWidgets.QLabel()
        img = os.path.join(self.dock.imageFolder,f"{coll.code}.png")
        ico = QtGui.QIcon(img)
        l.setPixmap(ico.pixmap(15,15))
        l.setMaximumWidth(20)
        hbox.addWidget(l)
        l = QtWidgets.QLabel(self.dock.param.get(coll.code))
        font = QtGui.QFont()
        font.setBold(True)
        l.setFont(font)
        hbox.addWidget(l)
        l = QtWidgets.QLabel()
        font = QtGui.QFont()
        font.setBold(True)
        l.setFont(font)
        hbox.addWidget(l)
        coll.infos = l
        cb = QtWidgets.QCheckBox("TOUS")
        cb.coll = coll
        coll.cbAll = cb
        cb.stateChanged.connect(partial(self.setAll, cb))
        layout.addWidget(cb)
        
    def makeLayerSelectList(self, layout, coll): 
        coll.excludes = {}
        for elt in self.dock.psql.getTableAdmin(coll.code, True):
            rdi = rdiLayer(elt)
            coll.addAbstract(rdi)
            if not rdi.tableName():
                continue
            name = rdi.layerName()
            if self.dock.param.get('blocDrop'):
                if elt['bloc'] and elt['bloc']!="":
                    name = f"[{elt['bloc']}] : {name}"
            cb = QtWidgets.QCheckBox(name)
            cb.rdi = rdi
            rdi.cb = cb
            if self.isExclude(rdi):
                coll.excludes[rdi.id()] = True
            else:
                cb.setChecked(True)
            layout.addWidget(cb)
            cb.stateChanged.connect(partial(self.setExclude, cb))
        l = QtWidgets.QLabel("")
        layout.addWidget(l)
   
    def setEnabled(self, bool):
        try:
            self.send.setEnabled(bool)
        except:
            pass
   
    def setAll(self, cb):
        bool = cb.isChecked()
        coll = cb.coll
        for rdi in coll.getElts():
            if not rdi.cb:
                continue
            rdi.cb.setChecked(bool)
    
    def setInfos(self, coll):
        tot = 0
        nb = 0
        for rdi in coll.getElts():
            if not rdi.cb:
                continue
            tot += 1
            if rdi.cb.isChecked():
                nb += 1
        try:
            l = coll.infos
            l.setText(f" ({nb}/{tot})")
            cb = coll.cbAll
            bool = (nb==tot)
            cb.blockSignals(True)
            cb.setChecked(bool)
            cb.blockSignals(False)
        except:
            pass
   
    def setExclude(self, cb):
        self.setEnabled(True)
        rdi = cb.rdi
        coll = rdi.coll
        self.setInfos(coll)
    
    def isExclude(self, rdi):
        return self.isExcludeId(rdi.id())
            
    def isExcludeId(self, id):
        if id in self.excludes.keys():
            return True

    def rebuiltList(self, coll):
        for rdi in coll.getElts():
            if not rdi.cb:
                continue
            cb = rdi.cb
            if not cb.isChecked():
                self.excludes[rdi.id()] = True
            else:
                try:
                    del self.excludes[rdi.id()]
                except:
                    pass
        
        self.setEnabled(False)
        self.dock.actuTabs(['dynamic'])




